@@ -1,4 +1,4 @@
-import StringIO
+import io
 import os
 import yaml
 import hepdata_converter
@@ -41,7 +41,6 @@ class ConvertTestSuite(WriterTestSuite):
             "---\n"
             "additional_resources: []\n"
             "data_file: data1.yaml\n"
-            "data_license: {description: null, name: null, url: null}\n"
             "description: 'The measured fiducial cross sections.  The first systematic uncertainty\n"
             "  is the combined systematic uncertainty excluding luminosity, the second is the luminosity.'\n"
             "keywords:\n"
@@ -100,7 +99,7 @@ class ConvertTestSuite(WriterTestSuite):
         self.assertRaises(ValueError, hepdata_converter.convert, None, None)
 
     def test_submission(self):
-        hepdata_converter.convert(StringIO.StringIO(self.simple_submission), self.current_tmp, options={'input_format': 'oldhepdata'})
+        hepdata_converter.convert(io.StringIO(self.simple_submission), self.current_tmp, options={'input_format': 'oldhepdata'})
 
         with open(os.path.join(self.current_tmp, 'submission.yaml')) as submission_file:
             self.assertEqual(list(yaml.load_all(submission_file, Loader=Loader)), list(yaml.load_all(self.correct_submit_output, Loader=Loader)))
